@@ -1,5 +1,5 @@
 # encoding=UTF-8
-# Copyright © 2008, 2009, 2010 Jakub Wilk <ubanus@users.sf.net>
+# Copyright © 2008, 2009, 2010 Jakub Wilk <jwilk@jwilk.net>
 #
 # This package is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -87,8 +87,8 @@ BBOX_RE = re.compile(
 BBOXES_RE = re.compile(
     r'''
         bboxes \s+
-        (         (?: \d+ \s+ \d+ \s+ \d+ \s+ \d+)
-        (?: , \s* (?: \d+ \s+ \d+ \s+ \d+ \s+ \d+) )* )
+        (          (?: -?\d+ \s+ -?\d+ \s+ -?\d+ \s+ -?\d+)
+        (?: ,? \s* (?: -?\d+ \s+ -?\d+ \s+ -?\d+ \s+ -?\d+) )* )
     ''', re.VERBOSE)
 
 class BBox(object):
@@ -221,12 +221,14 @@ class Zone(object):
             children=self.children,
         )
 
-def _replace_cuneiform_paragraph(paragraph, settings):
+def _replace_cuneiform08_paragraph(paragraph, settings):
     text = ''.join(
         ' ' if isinstance(character, Space) else character[0]
         for character in paragraph
     )
     if settings.details >= const.TEXT_ZONE_LINE:
+        # Cuneiform tends to attach superfluous whitespace:
+        text = text.rstrip()
         return [text]
     if len(text) != len(paragraph):
         raise ValueError
@@ -259,17 +261,21 @@ def _replace_text(djvu_class, title, text, settings):
         if text.endswith('\n'):
             embedded_eol = True
             text = text[:-1]
+    if settings.cuneiform:
+        # Cuneiform tends to attach superfluous whitespace:
+        text = text.rstrip()
     if settings.details >= djvu_class:
         return text,
     m = BBOXES_RE.search(title)
     if not m:
         return text,
-    coordinates = [map(int, bbox.strip().split()) for bbox in m.group(1).split(',')]
+    coordinates = (int(x) for x in m.group(1).replace(',', ' ').split())
+    coordinates = zip(coordinates, coordinates, coordinates, coordinates)
     if len(coordinates) == len(text):
         pass # OK
-    elif len(coordinates) != len(text):
+    else:
         if not embedded_eol and len(coordinates) == len(text) + 1:
-            # Ocropus produces weird hOCR output if line ends with a hyphen
+            # OCRopus produces weird hOCR output if line ends with a hyphen
             del coordinates[-1]
         else:
             raise errors.MalformedHocr("number of bboxes doesn't match text length")
@@ -343,7 +349,8 @@ def _scan(node, buffer, parent_bbox, settings):
             buffer += node.text,
     if not isinstance(node.tag, basestring):
         return
-    if settings.cuneiform:
+    if settings.cuneiform and settings.cuneiform <= (0, 8):
+        # Cuneiform ≤ 0.8 don't mark OCR elements in a hOCR way:
         djvu_class = cuneiform_tag_to_djvu(node.tag)
     else:
         hocr_classes = (node.get('class') or '').split()
@@ -352,6 +359,10 @@ def _scan(node, buffer, parent_bbox, settings):
             djvu_class = hocr_class_to_djvu(hocr_class)
             if djvu_class:
                 break
+        else:
+            if node.tag == 'p':
+                # Cuneiform ≥ 0.9 don't mark paragraphs in a hOCR way:
+                djvu_class = cuneiform_tag_to_djvu(node.tag)
     if not djvu_class:
         look_down(buffer, parent_bbox)
         return
@@ -362,6 +373,9 @@ def _scan(node, buffer, parent_bbox, settings):
     else:
         bbox = BBox(*(int(m.group(id)) for id in ('x0', 'y0', 'x1', 'y1')))
         parent_bbox.update(bbox)
+    if bbox.x0 == bbox.y0 == bbox.x1 == bbox.y1 == 0:
+        # Skip empty fragments generated by Cuneiform 0.9:
+        return
     if djvu_class is const.TEXT_ZONE_PAGE:
         if not bbox:
             if settings.page_size is None:
@@ -370,7 +384,7 @@ def _scan(node, buffer, parent_bbox, settings):
                     raise errors.MalformedHocr("cannot determine page's bbox")
                 settings.page_size = image_size.get_image_size(m.group('file_name'))
             page_width, page_height = settings.page_size
-            bbox = BBox(0, 0, page_width - 1, page_height - 1)
+            bbox = BBox(0, 0, page_width, page_height)
             parent_bbox.update(bbox)
         else:
             if (bbox.x0, bbox.y0) != (0, 0):
@@ -378,10 +392,19 @@ def _scan(node, buffer, parent_bbox, settings):
             settings.page_size = bbox.x1, bbox.y1
     result = Zone(type=djvu_class)
     look_down(result, bbox)
+    if bbox.x0 == bbox.y0 == bbox.x1 == bbox.y1 == 0:
+        # Skip empty fragments generated by Cuneiform 0.9:
+        return
     if len(result) > 0 and isinstance(result[-1], basestring):
+        if settings.cuneiform and settings.cuneiform == (0, 9):
+            # hOCR produced by Cuneiform 0.9 is really awkward, let's work
+            # around this.
+            bboxes_node = node.find('span[@class="ocr_cinfo"]')
+            if bboxes_node is not None and len(bboxes_node) == 0 and bboxes_node.text is None:
+                title = bboxes_node.get('title') or ''
         result[-1:] = _replace_text(djvu_class, title, result[-1], settings)
-    elif settings.cuneiform and djvu_class is const.TEXT_ZONE_PARAGRAPH:
-        result[:] = _replace_cuneiform_paragraph(result[:], settings)
+    elif settings.cuneiform and settings.cuneiform <= (0, 8) and djvu_class is const.TEXT_ZONE_PARAGRAPH:
+        result[:] = _replace_cuneiform08_paragraph(result[:], settings)
     if not bbox and not len(node):
         return
     if settings.page_size is None:
@@ -402,7 +425,7 @@ def scan(node, settings):
 
 class ExtractSettings(object):
 
-    def __init__(self, rotation=0, details=TEXT_DETAILS_WORD, uax29=None, page_size=None, cuneiform=False):
+    def __init__(self, rotation=0, details=TEXT_DETAILS_WORD, uax29=None, page_size=None, cuneiform=None):
         self.rotation = rotation
         self.details = details
         if uax29 is not None:
@@ -425,8 +448,12 @@ def extract_text(stream, **kwargs):
     settings = ExtractSettings(**kwargs)
     doc = ET.parse(stream, ET.HTMLParser())
     if doc.find('/head/meta[@name="ocr-capabilities"]') is None:
-        # Wild guess:
-        settings.cuneiform = True
+        ocr_system = doc.find('/head/meta[@name="ocr-system"]')
+        if ocr_system is not None and ocr_system.get('content') == 'openocr':
+            settings.cuneiform = (0, 9)
+        else:
+            # Wild guess:
+            settings.cuneiform = (0, 8)
     scan_result = scan(doc.find('/body'), settings)
     return [zone.sexpr for zone in scan_result]
 
